"""Script for running a tsunami inundation scenario for Broome, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a tsunami wave generated by MOST.

Ole Nielsen and Duncan Gray, GA - 2005 and Nick Bartzis, GA - 2006
"""

#-------------------------------------------------------------------------------
# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os
import time
from shutil import copy
from os import mkdir, access, F_OK
import sys

# Related major packages
from anuga.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.shallow_water.data_manager import convert_dem_from_ascii2netcdf, dem2pts
from anuga.abstract_2d_finite_volumes.combine_pts import combine_rectangular_points_files 
from anuga.geospatial_data.geospatial_data import *

# Application specific imports
import project                 # Definition of file names and polygons

#-------------------------------------------------------------------------------
# Define scenario as either slump or fixed_wave.
#-------------------------------------------------------------------------------
scenario = 'slump' # OR 'fixed_wave'
if access(scenario,F_OK) == 0:
    mkdir (scenario)
basename = scenario + 'source'

#-------------------------------------------------------------------------------
# Preparation of topographic data
#
# Convert ASC 2 DEM 2 PTS using source data and store result in source data
#-------------------------------------------------------------------------------

# filenames
dem_name = 'cairns' 
meshname = 'cairns.msh'

# creates DEM from asc data
convert_dem_from_ascii2netcdf(dem_name, use_cache=True, verbose=True)

#creates pts file for onshore DEM
dem2pts(dem_name, use_cache=True, verbose=True)

#----------------------------------------------------------------------------
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#-------------------------------------------------------------------------------

from anuga.pmesh.mesh_interface import create_mesh_from_regions
remainder_res = 10000000
islands_res = 100000
cairns_res = 100000
shallow_res = 500000
interior_regions = [[project.poly_cairns, cairns_res],
                    [project.poly_island0, islands_res],
                    [project.poly_island1, islands_res],
                    [project.poly_island2, islands_res],
                    [project.poly_island3, islands_res],
                    [project.poly_shallow, shallow_res]]

from caching import cache
_ = cache(create_mesh_from_regions,
          project.polyAll,
           {'boundary_tags': {'top': [0], 'ocean_east': [1],
                              'bottom': [2], 'onshore': [3]},
           'maximum_triangle_area': remainder_res,
           'filename': meshname,
           'interior_regions': interior_regions},
          verbose = True, evaluate=False)
print 'created mesh'

#-------------------------------------------------------------------------------                                 
# Setup computational domain
#-------------------------------------------------------------------------------                                 
domain = Domain(meshname, use_cache = True, verbose = True)

print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()
print domain.statistics()

domain.set_name(basename) 
domain.set_datadir(scenario)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
domain.set_minimum_storable_height(0.01)

#-------------------------------------------------------------------------------                                 
# Setup initial conditions
#-------------------------------------------------------------------------------

tide = 0.0
domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 
domain.set_quantity('elevation', 
                    filename = dem_name + '.pts',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1
                    )

#-------------------------------------------------------------------------------                                 
# Setup information for slump scenario (to be applied 1 min into simulation
#-------------------------------------------------------------------------------
if scenario == 'slump':
    from anuga.shallow_water.smf import slump_tsunami  # Function for submarine slump
    tsunami_source = slump_tsunami(length=15000.0,
                                   depth=project.slump_depth,
                                   slope=6.0,
                                   thickness=250.0, 
                                   radius=3330,
                                   dphi=0.23,
                                   x0=project.slump_origin[0], 
                                   y0=project.slump_origin[1], 
                                   alpha=0.0, 
                                   domain=domain,
                                   verbose=True)


#-------------------------------------------------------------------------------                                 
# Setup boundary conditions
#-------------------------------------------------------------------------------
print 'Available boundary tags', domain.get_boundary_tags()

Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])

# 7 min square wave starting at 1 min, 10m high
if scenario == 'fixed_wave':
    Bw = Time_boundary(domain = domain,
                       f=lambda t: [(60<t<480)*10, 0, 0])
    domain.set_boundary( {'ocean_east': Bw, 'bottom': Bd, 'onshore': Bd,
                           'top': Bd} )

# boundary conditions for slump scenario
if scenario == 'slump':
    domain.set_boundary( {'ocean_east': Bd, 'bottom': Bd, 'onshore': Bd,
                          'top': Bd} )


#-------------------------------------------------------------------------------                                 
# Evolve system through time
#-------------------------------------------------------------------------------
import time
t0 = time.time()

from Numeric import allclose
from anuga.abstract_2d_finite_volumes.quantity import Quantity

if scenario == 'slump':
    
    for t in domain.evolve(yieldstep = 30, finaltime = 60): 
        domain.write_time()
        domain.write_boundary_statistics(tags = 'ocean_east')     
        
        # add slump
        thisstagestep = domain.get_quantity('stage')
        if allclose(t, 60):
            slump = Quantity(domain)
            slump.set_values(tsunami_source)
            domain.set_quantity('stage', slump + thisstagestep)

        # save every two mins leading up to wave approaching land
        for t in domain.evolve(yieldstep = 120, finaltime = 1000, 
                               skip_initial_step = True):
            domain.write_time()
            domain.write_boundary_statistics(tags = 'ocean_east')

        # save every 30 secs as wave starts inundating ashore
        for t in domain.evolve(yieldstep = 30, finaltime = 10000, 
                               skip_initial_step = True):
            domain.write_time()
            domain.write_boundary_statistics(tags = 'ocean_east')

if scenario == 'fixed_wave':

    # save every two mins leading up to wave approaching land
    for t in domain.evolve(yieldstep = 120, finaltime = 1000): 
        domain.write_time()
        domain.write_boundary_statistics(tags = 'ocean_east')     

        # save every 30 secs as wave starts inundating ashore
        for t in domain.evolve(yieldstep = 30, finaltime = 10000, 
                               skip_initial_step = True):
            domain.write_time()
            domain.write_boundary_statistics(tags = 'ocean_east')
            
print 'That took %.2f seconds' %(time.time()-t0)
